# Generated by Django 5.2.3 on 2026-02-06 12:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "ichec_django_core",
            "0011_remove_memberpreferences_member_member_preferences",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="formfield",
            name="label",
            field=models.CharField(max_length=400),
        ),
        migrations.AlterField(
            model_name="formfield",
            name="options",
            field=models.CharField(max_length=1000, null=True),
        ),
        migrations.AlterField(
            model_name="formgroup",
            name="label",
            field=models.CharField(max_length=400, null=True),
        ),
    ]
