# Generated by Django 5.2.1 on 2026-02-27 14:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ichec_django_core", "0014_alter_member_options"),
    ]

    operations = [
        migrations.AddField(
            model_name="formfield",
            name="tooltip",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="formfield",
            name="description",
            field=models.TextField(blank=True, null=True),
        ),
    ]
