<html>
  <body>
    <h1>User Management Portal</h1>
    <p>You've been invited to join the User Managment Portal</p>

    <p>Follow this link to:<a href="{{action_url}}">sign up</a></p>

    <p>Thank you,</p>

    <p>User Management Portal Admin</p>
  </body>
</html>
