# Generated by Django 5.2.1 on 2026-02-20 17:53

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        (
            "ichec_django_core",
            "0012_alter_formfield_label_alter_formfield_options_and_more",
        ),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="member",
            options={
                "permissions": [("invite_member", "Can invite members")],
                "verbose_name": "Portal Member",
                "verbose_name_plural": "Portal Members",
            },
        ),
    ]
